package org.snmp4j.agent.db.sample;

import jetbrains.exodus.env.Environments;
import org.junit.Test;
import org.snmp4j.agent.DefaultMOServer;
import org.snmp4j.agent.MOServer;
import org.snmp4j.agent.db.MOXodusPersistence;
import org.snmp4j.agent.mo.snmp.UsmMIB;
import org.snmp4j.mp.MPv3;
import org.snmp4j.security.*;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.OctetString;
import org.snmp4j.util.ArgumentParser;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import static org.junit.Assert.*;

public class SampleAgentTest {

    public static final String AUTH_12345678 = "auth12345678";
    public static final String AUTH_876543210 = "auth876543210";
    public static final String PRIV_9876543210 = "priv9876543210";
    public static final String PRIV_123445678 = "priv123445678";

    private static final String[] ARGS = { "udp:127.0.0.1/4700"};

    private MOServer server;
    private MOXodusPersistence moXodusPersistence;
    private byte[] localEngineID;


    private SampleAgent sampleAgent;

    @org.junit.Before
    public void setUp() throws Exception {
        SecurityProtocols.getInstance().addDefaultProtocols();
        server = new DefaultMOServer();
        MOServer[] moServers = new MOServer[]{server};
        Path tempDir = Files.createTempDirectory("snmp4j-agent-db_unit-test");
        moXodusPersistence = new MOXodusPersistence(moServers, Environments.newInstance(tempDir.toFile()));

        ArgumentParser parser =
                new ArgumentParser(SampleAgent.COMMAND_LINE_OPTIONS, SampleAgent.COMMAND_LINE_PARAMS);
        Map<String, List<Object>> commandLineParameters;
        commandLineParameters = parser.parse(ARGS);

        sampleAgent = new SampleAgent(commandLineParameters);
    }

    @org.junit.After
    public void tearDown() throws Exception {

    }


    @Test
    public void resetConfig() {
        sampleAgent.run();
        localEngineID = sampleAgent.agent.getSnmpTargetMIB().getLocalEngineID();

        USM usm = sampleAgent.agent.getUsm();
        UsmMIB usmMIB = sampleAgent.agent.getUsmMIB();

        OctetString localEngineIDString = new OctetString(localEngineID);

        OctetString userNameAuthPriv = new OctetString("SHA256AES128");
        OctetString userNameAuthNoPriv = new OctetString("userAuthNoPriv");
        OctetString userName = new OctetString("user");

        AuthenticationProtocol authenticationProtocol = new AuthHMAC192SHA256();

        byte[] keyAuth = authenticationProtocol.passwordToKey(new OctetString(AUTH_12345678), localEngineID);
        byte[] keyPriv = authenticationProtocol.passwordToKey(new OctetString(PRIV_123445678), localEngineID);
        UsmUser usmUserAuthPriv = new UsmUser(userNameAuthPriv, AuthHMAC128SHA224.ID, new OctetString(keyAuth),
                PrivAES128.ID, new OctetString(keyPriv), localEngineIDString);
        UsmUser usmUserAuthNoPriv = new UsmUser(userNameAuthNoPriv, AuthHMAC128SHA224.ID, new OctetString(keyAuth),
                null, null, localEngineIDString);
        UsmUser usmUserNoAuthNoPriv =
                new UsmUser(userName, null, null, null, null, localEngineIDString);

        usm.addUser(usmUserAuthNoPriv);
        usm.addUser(usmUserAuthPriv);
        usm.addUser(usmUserNoAuthNoPriv);

        // Indexes Changed
        OID indexUsmUserAuthNoPriv =
                new OID(new OctetString(localEngineID).toSubIndex(false).getValue(),
                        userNameAuthNoPriv.toSubIndex(false).getValue());
        OID indexUsmUserAuthPriv =
                new OID(new OctetString(localEngineID).toSubIndex(false).getValue(),
                        userNameAuthPriv.toSubIndex(false).getValue());
        byte[] keyAuthNew = authenticationProtocol.passwordToKey(new OctetString(AUTH_876543210), localEngineID);
        byte[] keyPrivNew = authenticationProtocol.passwordToKey(new OctetString(PRIV_9876543210), localEngineID);

        sampleAgent.agent.saveState();

        // Check changed
        UsmMIB.UsmTableRow usmTableRowNew = usmMIB.getUsmUserEntry().getModel().getRow(indexUsmUserAuthPriv);
        assertNotNull(usmTableRowNew);
        assertArrayEquals(keyAuth, ((OctetString)usmTableRowNew.getValue(UsmMIB.colUsmUserAuthKey)).getValue());
        assertArrayEquals(keyPriv, ((OctetString)usmTableRowNew.getValue(UsmMIB.colUsmUserPrivKey)).getValue());
        assertEquals(userNameAuthPriv, usmTableRowNew.getValue(UsmMIB.colUsmUserSecurityName));

        usmTableRowNew = usmMIB.getUsmUserEntry().getModel().getRow(indexUsmUserAuthNoPriv);
        assertNotNull(usmTableRowNew);
        assertArrayEquals(keyAuth, ((OctetString)usmTableRowNew.getValue(UsmMIB.colUsmUserAuthKey)).getValue());
        assertEquals(userNameAuthNoPriv, ((OctetString)usmTableRowNew.getValue(UsmMIB.colUsmUserSecurityName)));
        

    }
}